"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const registry_data_1 = require("./registry-data");
const decorators_1 = require("./decorators");
class Container {
    constructor(parent) {
        this.parent = parent;
        this.registry = new Map();
    }
    register(provider) {
        provider = this.nornalizeProvider(provider);
        if (Array.isArray(provider)) {
            this.registerAll(provider);
        }
        else {
            provider = this.nornalizeProvider(provider);
            this.registerOne(provider);
        }
    }
    resolve(token) {
        const registryData = this.registry.get(token);
        if (!registryData) {
            if (this.parent) {
                return this.parent.resolve(token);
            }
            else {
                throw new Error(`No provider for ${token}`);
            }
        }
        if (registryData.value) {
            return registryData.value;
        }
        if (registryData.instance) {
            return registryData.instance;
        }
        if (registryData.factory) {
            let injections = [];
            if (registryData.injections) {
                injections = registryData.injections.map(i => this.resolve(i));
            }
            return registryData.factory(...injections);
        }
        const instance = this.createInstance(registryData);
        registryData.instance = instance;
        this.registry.set(token, registryData);
        return instance;
    }
    createScope() {
        return new Container(this);
    }
    registerAll(providers) {
        providers.forEach((p) => this.registerOne(p));
    }
    createInstance(registryData) {
        const cls = registryData.cls;
        const injectionsMd = this.getInjections(cls);
        const resolvedInjections = injectionsMd.map(injectionMd => this.resolve(injectionMd.token));
        const args = [];
        injectionsMd.forEach((injection, index) => {
            args[injection.parameterIndex] = resolvedInjections[index];
        });
        return new cls(...args);
    }
    registerOne(provider) {
        const registryData = new registry_data_1.RegistryData();
        if (provider.useValue) {
            registryData.value = provider.useValue;
        }
        else if (provider.useClass) {
            registryData.cls = provider.useClass;
        }
        else if (provider.useFactory) {
            registryData.factory = provider.useFactory;
            registryData.injections = provider.inject;
        }
        this.registry.set(provider.token, registryData);
    }
    nornalizeProvider(provider) {
        let normalizedProvider;
        if (Array.isArray(provider)) {
            normalizedProvider = provider.map((p) => this.normalizeOneProvider(p));
        }
        else {
            normalizedProvider = this.normalizeOneProvider(provider);
        }
        return normalizedProvider;
    }
    normalizeOneProvider(provider) {
        if (typeof provider === 'function') {
            provider = { token: provider, useClass: provider };
        }
        return provider;
    }
    getInjections(cls) {
        return Reflect.getOwnMetadata(decorators_1.INJECTIONS_MD_KEY, cls) || [];
    }
}
exports.Container = Container;
